import processing.opengl.*;
import processing.serial.*;
import toxi.geom.*;
import toxi.processing.*;

Serial myPort;
String myString = null;
int[] raw = new int[5];
int counter = 0;
float[] ypr = new float[3];

void setup() {

     size(1200, 600, OPENGL);

     loadTextureAttitudeIndicator();
     loadTextureDirectionalGyro();
    
     // setup lights and antialiasing
     lights();
     smooth(4);
    
     // display serial port list for debugging/clarity
     println(Serial.list());
   
     // open the serial port
     myPort = new Serial(this, Serial.list()[2], 57600);
     myPort.write("s");//s-115; e-101
  
     frameRate(60);
     background(0);
     ypr[0] = 0.; ypr[1] = 0.; ypr[2] = 0.;
}

void draw() {

     anime();
} //<>//

void serialEvent(Serial p) {

     myString = p.readStringUntil(0x0A);
     if (myString != null) {
        raw[counter] = Integer.parseInt(trim(myString));
        print(raw[counter]); print(" ");
        if (counter < 4) counter++;
        else { counter = 0; println(); }
     }//if
     doMPU6050();
     doHMC5883L();
}

void doMPU6050() {
float axD, ayD, azD;

     axD = (float)raw[0] / 16384.;
     ayD = (float)raw[1] / 16384.;
     azD = (float)raw[2] / 16384.;
     
     //ypr[2] = atan2(ayD, azD);
     ypr[2] = -atan(ayD / sqrt(axD * axD + azD * azD));
     ypr[1] = atan(-axD / sqrt(ayD * ayD + azD * azD));     
}

void doHMC5883L() {
float XmagF, YmagF;

     XmagF = (float)raw[3] / 1100.;
     YmagF = (float)raw[4] / 1100.;

     XmagF *= 100.;//SENSORS_GAUSS_TO_MICROTESLA;
     YmagF *= 100.;

     ypr[0] = atan2(YmagF, XmagF);
     //Magnetic declination at Sofia 4*38' = 0,08086692200907060373079767227331 rad
     //h_ttp://www.magnetic-declination.com/
     ypr[0] += 0.0809;
     if (ypr[0] < 0.) ypr[0] += 2.0 * PI;
     if (ypr[0] > 2.0 * PI) ypr[0] -= 2.0 * PI;
}

void anime() {
    
     pushMatrix();
        translate(width/4, height/2);
        scale(0.75);
        drawAttitudeIndicator(-ypr[2], -ypr[1]);
     popMatrix();

     //draw Directional Gyro
     pushMatrix();
        translate(3*width/4, height/2);
        scale(0.75);
        drawDirectionalGyro(-ypr[0]);
     popMatrix();
}