%Watson, M., The Design and Implementation of a Robust AHRS for Integration into a Quadrotor Platorm, MSc Thesis, University of Sheffield, May, 2013
%function [pitch, roll, bp, bq, P_out] = LKF(p, q, accelpitch, accelroll)
function [pitch, roll, bp, bq, P_out] = LKF(pitch,roll,p,q,dt)
Q_pitch = 500.0; % Calculated from data
Q_roll = 500.0; % Calculated from data
Q_bp = 0.1;
Q_bq = 0.1;
%dt = 0.025;
%pitch = zeros(length(p),1);%EKF,LKF
%roll = zeros(length(p),1);%EKF,LKF
bp = zeros(length(dt),1);%LKF
bq = zeros(length(dt),1);%LKF
P_out = zeros(length(dt),4,4);%LKF
R_pitch = 50; % Accel variance, calculated from data
R_roll = 50; % Accel variance, calculated from data
Q = [Q_pitch 0 0 0
     0 Q_roll 0 0
     0 0 Q_bp 0
     0 0 0 Q_bq];
R = [R_pitch 0 0 0
     0 R_roll 0 0
     0 0 0 0
     0 0 0 0];
P = [0 0 0 0
     0 0 0 0
     0 0 10000 0
     0 0 0 10000];
H = [1 0 0 0
     0 1 0 0
     0 0 0 0
     0 0 0 0];
I = [1 0 0 0
     0 1 0 0
     0 0 1 0
     0 0 0 1];
x = [pitch(1) % initial state estimate
     roll(1)
     0
     0];
for i = 2:1:length(dt)
    F = [1 0 -dt(i) 0
         0 1 0 -dt(i)
         0 0 1 0
         0 0 0 1];
    B = [dt(i) 0 0 0
         0 dt(i) 0 0
         0 0 0 0
         0 0 0 0];
    u = [p(i)
         q(i)
         0
         0];
    z = [pitch(i)
         roll(i)
         0
         0];
    % Predicted state estimate
    x = F*x + B*u;
    % Predicted estimate covariance
    P = F*P*F' + dt(i)*Q;
    % Measurement residual
    y = z - H*x;
    % Residual covariance
    S = H*P*H' + R;
    % Optimal Kalman gain
    K = P*H'*pinv(S);
    % Updated state estimate
    x = x + K*y;
    % Updated estimate covariance
    P = (I - K*H)*P;
    pitch(i) = x(1);
    roll(i) = x(2);
    bp(i) = x(3);
    bq(i) = x(4);
    P_out(i,1:4,1:4) = P(1:4,1:4);
end
end
