#include <MPU6050.h>

const int MPU6050 = 0xD0; //write address
const float rad2deg = 180.0 / 3.1415926535897932384626433832795;

void write8MPU6050(unsigned char address, unsigned char datum) {

     //Single byte write sequence
     I2C1_Start();     // Issue start signal
     I2C1_Wr(MPU6050); // Address + Write bit
     I2C1_Wr(address);
     I2C1_Wr(datum);
     I2C1_Stop();

     return;
}

void initMPU6050(void) {

     write8MPU6050(MPU6050_RA_PWR_MGMT_1, 0x00);   //Sleep OFF; Everything different than 0x01, i.e. 0x02
     write8MPU6050(MPU6050_RA_PWR_MGMT_2, 0x00);
     write8MPU6050(MPU6050_RA_ACCEL_CONFIG, 0x00); //accel_config +/-2g
     write8MPU6050(MPU6050_RA_GYRO_CONFIG, 0x00);  //gyro_config, +/-250 °/s

     return;
}

void readAccMPU6050(void) {

     //Burst read sequence
     while(!I2C1_Is_Idle());
     I2C1_Start();            // Issue start signal
     I2C1_Wr(MPU6050 & 0xFE); // Address + Write bit
     I2C1_Wr(0x3B);           // starting with register 0x3B
     I2C1_Repeated_Start();
     I2C1_Wr(MPU6050 | 0x01);
     AccX = I2C1_Rd(1u) << 8 | I2C1_Rd(1u); // 0x3B (ACCEL_XOUT_H) & 0x3C (ACCEL_XOUT_L)
     AccY = I2C1_Rd(1u) << 8 | I2C1_Rd(1u); // 0x3D (ACCEL_YOUT_H) & 0x3E (ACCEL_YOUT_L)
     AccZ = I2C1_Rd(1u) << 8 | I2C1_Rd(0u); // 0x3F (ACCEL_ZOUT_H) & 0x40 (ACCEL_ZOUT_L)
     I2C1_Stop();

     return;
}

void readGyrMPU6050(void) {

     //Burst read sequence
     while(!I2C1_Is_Idle());
     I2C1_Start();            // Issue start signal
     I2C1_Wr(MPU6050 & 0xFE); // Address + Write bit
     I2C1_Wr(0x43);           // starting with register 0x43
     I2C1_Repeated_Start();
     I2C1_Wr(MPU6050 | 0x01);
     GyrX = I2C1_Rd(1u) << 8 | I2C1_Rd(1u); // 0x43 (GYRO_XOUT_H) & 0x44 (GYRO_XOUT_L)
     GyrY = I2C1_Rd(1u) << 8 | I2C1_Rd(1u); // 0x45 (GYRO_YOUT_H) & 0x46 (GYRO_YOUT_L)
     GyrZ = I2C1_Rd(1u) << 8 | I2C1_Rd(0u); // 0x47 (GYRO_ZOUT_H) & 0x48 (GYRO_ZOUT_L)
     I2C1_stop();

     return;
}

void readTmpMPU6050(void) {

     //Burst read sequence
     while(!I2C1_Is_Idle());
     I2C1_Start();            // Issue start signal
     I2C1_Wr(MPU6050 & 0xFE); // Address + Write bit
     I2C1_Wr(0x41);           // starting with register 0x41
     I2C1_Repeated_Start();
     I2C1_Wr(MPU6050 | 0x01);
     Tmp  = I2C1_Rd(1u) << 8 | I2C1_Rd(0u); // 0x41 (TEMP_OUT_H) & 0x42 (TEMP_OUT_L)
     I2C1_stop();

     return;
}
