#include <MPU6050.h>
#include <BMP085.h>
#include <HMC5883L.h>

unsigned char data_[8];
int AccX, AccY, AccZ, GyrX, GyrY, GyrZ, Tmp;
signed long int UT, UP;
signed int Xmag, Ymag, Zmag;

void genericInit() {

     TRISB = 0;
     CMCON = 0x07;           // Disable comparators
     ADCON1 = 0b00001111;    // All digital
     ADCON0 = 0b00101100;
     UCON.USBEN = 0;
     SSPCON1.SSPEN = 1;      // SDA e SCL porta serial

     UART1_Init(57600);//57600 for 3DR
     UART_Set_Active(&UART1_Read, &UART1_Write, &UART1_Data_Ready, &UART1_Tx_Idle); // set UART1 active

     return;
}

void initTimer0(void) {

     //Timer0
     //Prescaler 1:128; Actual Interrupt Time : 1,0000128 s
     T0CON = 0b00000110; // Timer0 is stopped

//     INTCON.GIEH = 0; // Forbidden if one transfers data through USB
     INTCON.TMR0IE = 0;
     INTCON.TMR0IF = 0;

     TMR0H = 0x67;
     TMR0L = 0x69;

     return;
}

void startTimer0(void) {

     T0CON.TMR0ON = 1; // Start the timer

     return;
}

float stopTimer0(void) {
unsigned int timerValue = 0;
unsigned char* timerValuePtr = 0;
unsigned int resultL, resultH;
float result;

      T0CON.TMR0ON = 0; // Stop the timer
      timerValuePtr = (unsigned char*)&timerValue;

      //Read TMR0L first!
      *timerValuePtr = TMR0L;
      resultL = *timerValuePtr;
//      WordToStr(*timerValuePtr, &data_[0]);
      timerValuePtr++;
      *timerValuePtr = TMR0H;
      resultH = *timerValuePtr;
//      WordToStr(*timerValuePtr, &data_[8]);

      // 0x6769 = 0d26473
      TMR0H = 0x67;
      TMR0L = 0x69;

      // 65535 - 26473 = 1000 ms = 39062
      result = (float)((resultH << 8) | resultL) + 0.005 - 26473.;
      result *= 1000. / 39062.;//ms

      return result;
}

void sendThroughUARTtoMSVS(void) {

     IntToStr(AccX, data_);
     UART1_Write_Text(data_);
     UART1_Write(' ');
     IntToStr(AccY, data_);
     UART1_Write_Text(data_);
     UART1_Write(' ');
     IntToStr(AccZ, data_);
     UART1_Write_Text(data_);
     UART1_Write(' ');
     IntToStr(GyrX, data_);
     UART1_Write_Text(data_);
     UART1_Write(' ');
     IntToStr(GyrY, data_);
     UART1_Write_Text(data_);
     UART1_Write(' ');
     IntToStr(GyrZ, data_);
     UART1_Write_Text(data_);
     UART1_Write(' ');
     IntToStr(Tmp, data_);
     UART1_Write_Text(data_);
     UART1_Write(' ');
     
     IntToStr(UT, data_);
     UART1_Write_Text(data_);
     UART1_Write(' ');
     LongWordToStr(UP, data_);
     LTrim(data_);
     UART1_Write_Text(data_);

     IntToStr(Xmag, data_);
     UART1_Write_Text(data_);
     UART1_Write(' ');
     IntToStr(Ymag, data_);
     UART1_Write_Text(data_);
     UART1_Write(' ');
     IntToStr(Zmag, data_);
     UART1_Write_Text(data_);
     UART1_Write(' ');
     
//     UART1_Write(13);//CR
     UART1_Write(31);//US

     return;
}

void sendThroughUARTtoProcessing(void) {

     IntToStr(AccX, data_);
     UART1_Write_Text(data_);
     UART1_Write(0x0A);
     IntToStr(AccY, data_);
     UART1_Write_Text(data_);
     UART1_Write(0x0A);
     IntToStr(AccZ, data_);
     UART1_Write_Text(data_);
     UART1_Write(0x0A);

     IntToStr(Xmag, data_);
     UART1_Write_Text(data_);
     UART1_Write(0x0A);
     IntToStr(Ymag, data_);
     UART1_Write_Text(data_);
     UART1_Write(0x0A);

     return;
}
/*
void sendCoefThroughUART(void) {
signed int AC1, AC2, AC3, AC4, AC5, AC6, B1, B2, MB, MC, MD;

     AC1 = coefficients(BMP085_CAL_AC1);
     IntToStr(AC1, data_);
     UART1_Write_Text(data_);
     UART1_Write(' ');
     
     AC2 = coefficients(BMP085_CAL_AC2);
     IntToStr(AC2, data_);
     UART1_Write_Text(data_);
     UART1_Write(' ');

     return;
}
*/
void main() {

     genericInit();
     I2C1_Init(100000);      // Initiate I2C @ 100 kHz
     initMPU6050();
     //CallibrateMPU6050raw(); // Measures the offset values of both accelerometer and gyroscope
     initHMC5883L();
/*
     initTimer0();
     startTimer0();
     Delay_ms(1000);         // Maximum allowable time
     stopTimer0();
*/

     Delay_ms(100);                   // Wait for UART module to stabilize
     
     while(1) {
        do {} while(UART1_Read() != 's'); // Wait until start signal is received
        do {
           readAccMPU6050();
           readGyrMPU6050();
           readTmpMPU6050();
           readUTBMP085();
           readUPBMP085();
           readHMC5883Lraw();
           sendThroughUARTtoMSVS();
           //Delay_ms(1000);
        } while(UART1_Read() != 'e'); // Do until end signal is received
     }//while

     return;
}